/*
 * This file is part of the MGUI (Morello Graphic User Interface)
 * library. Copyright 1995-2003 Vincenzo Morello.
 *
 * Contains all MGUI macro definitions and typedefs
 */

#ifndef _MGUIDEFS_H
#define _MGUIDEFS_H

#define MGUI_RELEASE    0x412

/*
 * Auto environment detection (if not set):
 * BC_MGUI (DOS BGI), DJGPP_MGUI, W4GW_MGUI, WGRX_MGUI and WIN_MGUI
 * (both 16 & 32 bit) are detected.
 * WIN32 and __WIN32__ are set when compiling for Win32, Win 95 and NT.
 * X11_MGUI is set by default. LIBGRX must be set to compile in LINUX
 * console (svgalib)
 */

#if !defined DJGPP_MGUI && !defined X11_MGUI && !defined BC_MGUI && !defined WIN_MGUI && !defined WGRX_MGUI && !defined W4GW_MGUI
        #if defined __TURBOC__
                #if defined __DPMI32__
                        #define BGI_MGUI
                #elif defined __DPMI16__
                        #define BGI_MGUI
                #elif defined _Windows
                        #define WIN_MGUI
                        #if defined __WIN32__ && !defined WIN32
                                #define WIN32
                        #endif
                #else
                        #define BGI_MGUI
                        #define BC_MGUI
                #endif
        #elif defined __CYGWIN32__
                #define WIN_MGUI
                #define WIN32
                #define __WIN32__
                #define __GNUWIN32__
        #elif defined __MINGW32__
                #define WIN_MGUI
                #if !defined WIN32
                                #define WIN32
                #endif
                #define __GNUWIN32__
        #elif defined __LCC__
                #define WIN_MGUI
                #define __WIN32__
        #elif defined __RSXNT__
                #define WIN_MGUI
                #define WIN32
                #define __GNUWIN32__
 	#elif defined __GO32__
                #if defined __WIN32__
                        #ifndef WIN32
		                #define WIN32
	                #endif
                        #define WIN_MGUI
                #else
                        #define DJGPP_MGUI
                        #define LIBGRX
                #endif
        #elif defined _WINDOWS
                #define WIN_MGUI
                #ifdef _WINNT
                        #define __WIN32__
                #endif
        #elif defined __WATCOMC__
                #ifdef __NT__
                        #define __WIN32__
                        #define WIN_MGUI
                #elif defined __WINDOWS__
                        #define WIN_MGUI
                #elif defined LIBGRX
                        #define WGRX_MGUI
                #elif defined METAWINDOW
                        #define WMW_MGUI
                #else 
                        #define W4GW_MGUI
                #endif
        #else
                #ifndef LIBGRX
                        #define X11_MGUI
                #endif
                #define UNIX
        #endif
#endif

#if defined __TURBOC__ || defined _MSC_VER || defined __WATCOMC__ || defined __LCC__
#include <time.h>
#elif !defined WIN_MGUI
#include <sys/time.h>
#endif

/*
 * This include file contains key codes definition
 */
#include "keycodes.h"

#ifdef UNIX
#define DIR_SEP   '/'
#else
#define DIR_SEP   '\\'
#endif

#if defined __GNUC__ && !defined WIN32
#define stricmp strcasecmp
#endif

/**
 *	\defgroup structures MGui Structures
 */
/**
 *	\defgroup macros Macros
 */

 /** \ingroup structures 
 \brief Point structure.

Used as argument of polyline drawing primitives.
See MDrawPolyline(), MDrawFilledPolyline(),
CmObject::drawPolyline, CmObject::drawFilledPolyline
 */
struct MPOINT {
#if defined X11_MGUI
	short x;
	short y;
#else
	int x; // point's x coordinate
	int y; // point's y coordinate
#endif
};
typedef struct MPOINT MPOINT;

/*
 * Integer size typedefs
 */
#ifdef X11_MGUI
#include "X11/X.h"
#include <X11/Xmd.h>
#else
#if !defined _BASETSD_H_ && !defined _BASETSD_H
        typedef long            INT32;
#endif
        typedef short           INT16;
        typedef char            INT8;
        typedef unsigned long   CARD32;
        typedef unsigned short  CARD16;
        typedef unsigned char   CARD8;
#endif /* X11_MGUI */

/**
 Font identifier
 */
typedef int MTFont;

/**
 Mouse cursor identifier
 */
typedef int MTCursor;

/** Color identifier */
typedef CARD32 MTColor;

/* Transparent color definition */
#define NOCOLOR	((MTColor)~0)

#if !defined _MGUI_OBJECT_H
/**
  Opaque MGui object identifier.
 */
typedef void *MOBJECT;

#endif

/** Rectangle structure (for internal use) */
struct MRECT {
	int x;
	int y;
	int w;
	int h;
};
typedef struct MRECT MRECT;

/** Rectangle list structure (for internal use) */
struct RLIST {
	MRECT r;
	struct RLIST *next;
};
typedef struct RLIST RLIST;

/*
 * Window type definition.
 * Window typedef form X11 is used as is (not redefined).
 */
#if !defined(X11_MGUI) && !defined(WIN_MGUI)

/* Window structure for DOS and Linux/console environments */
struct _Window { MRECT r; RLIST *rl; };

/* Window identifier for DOS and Linux/console environments */
typedef struct _Window *Window;

#elif defined WIN_MGUI

/* Window identifier under Windows */
typedef void *Window;

#endif



#ifndef True

#define True    1
#define False   0

#endif

/** \ingroup macros
 * \name Color Brightness percentages.
 * These macros are internally used by the MGui library for brightness percentage
 * levels.
 * Using predefined values for color shading is recommended especially when the
 * program runs in Palette mode (up to 256 colors).
 * These values can be used as the percentage parameter in functions like
 * MAllocShiftColor(), MScrolledFormSetSliderColorShift(),
 * MScrollBarSetSliderColorShift(), MSListSetSliderColorShift(),
 * MScrolledEditSetSliderColorShift(), MTreeSetSliderColorShift(),
 * MObjectSetBGColorFromParent()
 */
//@{
/// for object border shadows
#define CS_LIGHTER	140	
/// for object border shadows
#define CS_DARKER	60
/// for armed button bg and default selection highlights
#define CS_DARK		85
/// for default selection highlights
#define CS_LIGHT	115
//@}

/** \ingroup macros
* \name MGUI Events
* Events generated by the MGui library have one of the following values
* in the \e type member of all structures in the MEvent union.
*/
//@{
#define E_NO_EVENT              0
#define E_KEY_PRESS             1
#define E_BUTTON_PRESS          2
#define E_BUTTON_RELEASE        3
#define E_MOTION                4
#define E_WINDOW_ENTER          5
#define E_WINDOW_EXIT           6
#define E_EXPOSE                7
#define E_DESTROY               8
#define E_MAP                   9
#define E_UNMAP                 10
#define E_AUTOSIZE              11
#define E_RESIZE                12
#define E_MOVE                  13
#define E_SELECT                14
#define E_DESELECT              15
#define E_CONFIGURE             16
#define E_PARENT_MAP            17
#define E_PARENT_UNMAP          18
#define E_SET_COLOR             19
#define E_SET_SHADOW            20
#define E_SET_TEXT              21
#define E_GET_TEXT              22
#define E_WM_CLOSE              23
#define E_SET_SENS              24
#define E_SET_PARENT_SENS       25
#define E_GRAB_LOSE             26
#define E_SET_BG_IMAGE          27
#define E_MOUSE_WHEEL           28
#define E_SET_TRANSPARENCY      29
#define E_PARENT_CHANGED        30
//@}

/** \ingroup macros
 * \name Event Masks.
 * Bitmask values to select a set of events in a single variable.
 * A bitmask of these values can be used to set the desired event types to
 * be reported in callbacks set via MObjectAddEventNotifyCallback(), 
 * CmObject::setEventCallback.
 */
//@{
#define EM_NO_EVENT             0L
#define EM_KEY_PRESS            1L
#define EM_BUTTON_PRESS     (1L<<1)
#define EM_BUTTON_RELEASE   (1L<<2)
#define EM_MOTION           (1L<<3)
#define EM_WINDOW_ENTER     (1L<<4)
#define EM_WINDOW_EXIT      (1L<<5)
#define EM_EXPOSE           (1L<<6)
#define EM_DESTROY          (1L<<7)
#define EM_MAP              (1L<<8)
#define EM_UNMAP            (1L<<9)
#define EM_AUTOSIZE         (1L<<10)
#define EM_RESIZE           (1L<<11)
#define EM_MOVE             (1L<<12)
#define EM_SELECT           (1L<<13)
#define EM_DESELECT         (1L<<14)
#define EM_CONFIGURE        (1L<<15)
#define EM_PARENT_MAP       (1L<<16)
#define EM_PARENT_UNMAP     (1L<<17)
#define EM_SET_COLOR        (1L<<18)
#define EM_SET_SHADOW       (1L<<19)
#define EM_SET_TEXT         (1L<<20)
#define EM_GET_TEXT         (1L<<21)
#define EM_WM_CLOSE         (1L<<22)
#define EM_SET_SENS         (1L<<23)
#define EM_SET_PARENT_SENS  (1L<<24)
#define EM_GRAB_LOSE        (1L<<25)
#define EM_SET_BG_IMAGE     (1L<<26)
#define EM_MOUSE_WHEEL      (1L<<27)
#define EM_SET_TRANSPARENCY (1L<<28)
#define EM_PARENT_CHANGED   (1L<<29)
//@}

#if !defined X11_MGUI || defined DOXYGEN
/** \ingroup macros
 * \name Mouse button bitmask in mouse events.
 * These values can be found in the \e state member of structure MMouseEvent.
 * By masking the \e state field with these macro values you can realize which
 * mouse buttons were hold down when the event was generated.
 */
//@{
#define Button1Mask             (1<<8)
#define Button2Mask             (1<<9)
#define Button3Mask             (1<<10)
//@}

/** \ingroup macros
 * \name Mouse button bitmask in mouse events.
 * Only one of these values can be found in the \e button field of the MMouseEvent
 * structure. Note that the given button can be pressed or released depending on
 * the actual value of the \e type member in the same event (E_BUTTON_PRESS or
 * E_BUTTON_RELEASE)
 */
//@{
#define Button1                 1
#define Button2                 2
#define Button3                 3
//@}

/** \ingroup macros
 * \name Keyboard Modifier key masks.
 * By masking the value o these macros with the \e state field in events MKeyEvent
 * and MMouseEvent you can figure out the hold down status of such keys at the time
 * the event was generated. You can also mask them with the value returned by MGetKeyboardState
 * to check the status of the modifier keys at any time.
 */
//@{
#define ShiftMask       (1<<0)
#define LockMask        (1<<1)
#define ControlMask     (1<<2)
#ifdef DOXYGEN
#define AltMask         (1<<3)
#endif
//@}

#endif

#define AltMask         (1<<3)

/**
  Pointer to the image raw pixel data.
 */
#if defined BC_MGUI || defined __DPMI16__ || defined WIN_MGUI && !defined __WIN32__ && !defined _MSC_VER
typedef unsigned char huge *IMAGE_DATA;
#else
typedef unsigned char *IMAGE_DATA;
#endif

/** \ingroup structures 
 \brief Colormap entry.

   A colormap is associated to an image with a fixed limited number of colors (usually 2, 16 or 256).
   An array of IMAGE_CMAP entries is always bound to an MIMAGE structure with an
   associated color palette.
 */
struct IMAGE_CMAP {
		/** Blue color component */
	unsigned char blue;
		/** Green color component */
	unsigned char green;
		/** Red color component */
	unsigned char red;
		/** Palette index.
		This field is intended for internal use.
		When the program runs in PaletteMode (nr of colors <= 256) this field
		stores the color index corresponding to the palette entry when the image
		is installed.
		*/
	unsigned char index;
};
typedef struct IMAGE_CMAP IMAGE_CMAP;

/** \ingroup structures 
 \brief Image description.

  This is the structure MGui uses to handle images. 
  \ref MIMAGE, a pointer to this structure, should be used by the programmer
  in order to treat the image as an opaque object.
  See \ref image and CmImage
  for available functions/methods.
 */
struct MIMAGE_DESCR {
				/** Boolean flag describing whether image colors
				 have been allocated or not */
        unsigned int installed;
				/** Number of colors in the image (0 if TrueColor */
        unsigned int ncolors;
				/** Number of planes in the image (1,4,8,24) */
        unsigned int nplanes;
				/** Image width in pixel units */
        unsigned int img_w;
				/** Image height in pixel units */
        unsigned int img_h;
				/** Bytes per row */
        unsigned int row_w;
				/** Low level pixmap id (system dependent) */
        CARD32 pixmap_id;
				/** Image colormap (NULL for 24 planes images) */
        IMAGE_CMAP *colormap;
				/** Current Image raw pixel data */
        IMAGE_DATA data;
				/** Backup Image raw pixel data */
        IMAGE_DATA backup_data;
				/** Image alpha channel data */
        IMAGE_DATA alpha;
				/** Image file name (if loaded from file) */
        char *fname;
				/** Image mask */
		struct MIMAGE_DESCR *mask;
				/** Internal use only */
        CARD32 flags;
};
typedef struct MIMAGE_DESCR MIMAGE_DESCR;

typedef MIMAGE_DESCR *MIMAGE;

/**	\defgroup events MGui Events
 * MGui events are: mouse and keyboard input, window expose and resize,
 * or internally generated events such as 'text change', 'color change'
 * or 'shadow change'. 
 * Many MGui events are gathered from the underlying windowing system
 * (the Window Manager). 
 * Console versions of MGui (DOS and Linux console mode) generate events
 * inside an MGui module which emultates the Window Manager.
 * System generated events include:
 * - \ref MKeyEvent
 * - \ref MMouseEvent
 * - \ref MExposeEvent
 * - \ref MWMCloseEvent
 * - \ref MFocusEvent
 * - \ref MUnmapEvent
 * - \ref MMapEvent
 * - \ref MConfigureEvent
 * - \ref MMoveEvent
 * - \ref MResizeEvent
 *
 * Other events are used internally by the library to notify objects that their
 * properties have been (are being) changed.
 They are not related to the Window Manager.
 * - \ref MSetColorEvent
 * - \ref MSetShadowEvent
 * - \ref MSetGetTextEvent
 * - \ref MSetSensitivity
 * - \ref MSetBGImageEvent
 * - \ref MSetTransparencyEvent
 */
//@{

/**
   Generic Event.
   This structure contains the fields common to all events generated
   by the windowing system.
   Many MGui events are defined as typedef to this event:
   \ref MWMCloseEvent, \ref MFocusEvent, \ref MUnmapEvent, \ref MMapEvent
 */
struct MAnyWindowEvent {
/// Event type
	int type;
/// Window which the event is directed to
	Window window;
};
typedef struct MAnyWindowEvent MAnyWindowEvent;

/**
 * Keyboard event.
 * The windowing system generates this event as the user presses a key in the
 * keyboard. The event is bound, via field \e window, to the top level window
 * of the application currently owning the input focus.
 * The event is passed to the SHELL object bound to the focused window
 * and then routed to the MGui object that actually owns the input focus.
 *
 * It's possible to send directly this event to an MGui object, via
 * MDispatchObjEvent() or CmObject::sendEvent, to simulate
 * key presses forcing an object to act consequently. In this case, the \e window
 * field is not relevant and can be set to anything. Other fields should instead
 * be set to correct values.
 */
struct MKeyEvent {
/// Event type (E_KEY_PRESS)
	int type;
/// Window which the event is directed to
	Window window;
/// Bitmask of the state of modifier keys (ShiftMask, LockMask, ControlMask, AltMask)
	unsigned int state;
/// Code for the pressed key
	unsigned int keycode;
};
typedef struct MKeyEvent MKeyEvent;
/**
 * \brief Window Close Event.
 *
 * The Window Manager uses this event to notify the program that a given
 * window must be closed.
 * The \e type field is E_WM_CLOSE.
 */
typedef MAnyWindowEvent MWMCloseEvent;

/**
 * \brief Window/Object Focus Event.
 *
 * The Window Manager uses this event to notify the program that a given
 * window is gaining/losing the input focus.
 * The value of the \e type field is either E_SELECT or E_DESELECT.
 * The same event is then propagated by the library to the MGui object
 * actually gaining/losing the keyboard focus.
 */
typedef MAnyWindowEvent MFocusEvent;

/**
 * \brief Window Unmap Event.
 *
 * The Window Manager uses this event to notify the program that a given
 * window has been removed from the screen.
 * The \e type field is either E_UNMAP
 * The event is processed the SHELL object and then propagated to all MGui objects
 * inside the window being unmapped.
 * \sa MObjectUnmap(), CmObject::unmap
 */
typedef MAnyWindowEvent MUnmapEvent;

/**
 * \brief Window/Object Map Event.
 *
 * The Window Manager uses this event to notify the program that a given
 * window has been opened on the screen.
 * The value of the \e type field is E_MAP
 * The event is processed the SHELL object and then propagated to all MGui objects
 * inside the window being mapped.
 * \sa MObjectMap(), CmObject::map
 */
typedef MAnyWindowEvent MMapEvent;

/*
 * Input Focus Gaining Event.
 * The Window Manager uses this event to notify the program that a given
 * window is gaining the input focus. The same event is then propagated
 * by the library to the MGui object actually gaining the keyboard focus.
 */
typedef MAnyWindowEvent MSelectEvent;

/*
 * Input Focus Losing Event.
 * The Window Manager uses this event to notify the program that a given
 * window is losing the input focus. The same event is then propagated
 * by the library to the MGui object actually gaining the keyboard focus.
 */
typedef MAnyWindowEvent MDeSelectEvent;

/**
 * Mouse event.
 * This structure is used to notify the application about four type of events
 * related to mouse (E_BUTTON_PRESS, E_BUTTON_RELEASE, E_MOTION, E_MOUSE_WHEEL).
 * You can get these events as callback parameter
 */
struct MMouseEvent {
/// Event type (E_BUTTON_PRESS, E_BUTTON_RELEASE, E_MOTION, E_MOUSE_WHEEL)
	int type;
/// Window which the event is directed to
	Window window;
/// pointer x, y coordinates in event window
	int x, y;
/// coordinates relative to root
	int x_root, y_root;
/// buttons and key modifiers mask
	unsigned int state;
/// button pressed/released (Button1, Button2 or Button3), 0 if no button involved
	unsigned int button;
/// scrolling lines (mouse wheel command)
	int scroll_delta;
};
typedef struct MMouseEvent MMouseEvent;

/**
 * \brief Window/Object Expose Event.
 *
 * The Window Manager uses this event to notify the program that a portion of
 * a given window needs to be redrawn.
 * The value of the \e type field is E_EXPOSE
 * The event is processed the SHELL object and then propagated to all MGui objects
 * inside the area that needs to be redrawn.
 * \sa MObjectRedraw(), CmObject::redraw
 */
struct MExposeEvent {
/// Event type (E_EXPOSE)
        int type;
/// Window which the event is directed to
        Window window;
/// offset of the area to redraw
        int x_off, y_off;
/// coordinates of the upper left corner of the area to redraw
        int x, y;
/// Size of the area to redraw
        int width, height;
/// Flag stating whether the parent object has already drawn itself
        int parent_drawn;
};
typedef struct MExposeEvent MExposeEvent;

/**
 * \brief Window/Object Resize Event.
 *
 * The Window Manager uses this event to notify the program that a 
 * a given window has been resized.
 * The value of the \e type field is E_RESIZE
 * The event is processed the SHELL object and then propagated to all MGui objects
 * inside the area that needs to be resized.
 * \sa MObjectResize(), CmObject::resize
 */
struct MResizeEvent{
/// Event type (E_RESIZE)
        int type;
/// Window which the event is directed to
        Window window;
/// New Window/Object size
        int width, height;
};
typedef struct MResizeEvent MResizeEvent;

/**
 * \brief Window Configure Event.
 *
 * The Window Manager uses this event to notify the program that a 
 * a given window has been resized and/or moved.
 * The \e type field is E_CONFIGURE
 * The event is processed the SHELL object only.
 * \sa MShellConfigure(), CmShell::configure
 */
typedef struct {
/// Event type (E_CONFIGURE)
        int type;
/// Window which the event is directed to
        Window window;
/// New Window position (including decorations)
        int x, y;
/// Window offset (relative to decorated parent window)
        int x_off, y_off;
/// New Window size
        int width, height;
} MConfigureEvent;

/**
 * \brief Window Move Event.
 *
 * The Window Manager uses this event to notify the program that a 
 * a given window has been moved.
 * The \e type field is E_MOVE
 * The event is processed the SHELL object only.
 * \sa MShellConfigure(), CmShell::configure
 */
typedef struct {
/// Event type (E_MOVE)
        int type;
/// Window which the event is directed to
        Window window;
/// New Window position
        int x, y;
} MMoveEvent;

/* misc events (related to Objects rather than windows). */

/**
 * \brief Object Set Sensitivity Event.
 *
 * This event is generated by the library to notify an object that its 
 * sensitivity status need to be changed.
 * The event is generated by one any of the following functions/methods:
 * MObjectSetSensitive(), MObjectSetUnsensitive(), MObjectSetSensitivity(),
 * CmObject::setSensitive, CmObject::setUnsensitive, CmObject::setSensitivity.
 * The value for the \e type field is set initially to E_SET_SENS. If the addressed
 * object is a container (SHELL or FORM) then the object notifies all its children
 * that its sensitivity has changed. In this case, the \e type field is set to
 * E_SET_PARENT_SENS.
 */
typedef struct {
/// Event type (E_SET_SENS, E_SET_PARENT_SENS)
	int type;
/// Boolean flag representing the new sensitivity status.
	int sensitive;
} MSetSensitivity;

/**
 * \brief Object Set/Get Text Event.
 *
 * This event is generated by the library to set/get the object's text.
 * The event is generated by the following functions/methods:
 * MObjectSetText(), MObjectGetText(),
 * CmObject::setText, CmObject::getText.
 */
typedef struct {
/// Event type (E_SET_TEXT, E_GET_TEXT)
        int type;
/// Pointer to the string to read text from (to store text to).
        char *text;
} MSetGetTextEvent;

/**
 * \brief Object Set Color Event.
 *
 * This event is generated by the library to set the object's background
 * and/or foreground colors.
 * The event is generated by the following functions/methods:
 * MObjectSetColor(), MObjectSetBackgroundColor(), MObjectSetForegroundColor(),
 * MObjectSetBackgroundRGB(), MObjectSetForegroundRGB(),
 * CmObject::setColor, CmObject::setBackgroundColor, CmObject::setForegroundColor,
 * CmObject::setBackgroundRGB, CmObject::setForegroundRGB
 */
typedef struct {
/// Event type (E_SET_COLOR)
	int type;
/// Background color
	MTColor bgc;
/// Foreground color
	MTColor fgc;
} MSetColorEvent;

/**
 * \brief Object Set Shadow Event.
 *
 * This event is generated by the library to set the object's shadow
 * property.
 * The event is generated by:
 * MObjectSetShadow(), CmObject::setShadow
 */
typedef struct {
/// Event type (E_SET_SHADOW)
	int type;
/// Shadow type
	int shadow;
/// Internal border thickness
	int in_thick;
/// External border thickness
	int out_thick;
} MSetShadowEvent;

/**
 * \brief Object Set background image Event.
 *
 * This event is generated by the library to set the object's background image
 * property.
 * The event is generated by:
 * MObjectSetBackgroundImage(), MObjectSetBackgroundImageFile(),
 * CmObject::setBackgroundImage, CmObject::setBackgroundImageFile
 */
typedef struct {
/// Event type (E_SET_BG_IMAGE)
	int type;
/// Background image
	MIMAGE img;
/// Image drawing mode (BI_TILED, BI_CENTERED, BI_SIZE_FORCING)
	CARD8  mode;
} MSetBGImageEvent;

/**
 * \brief Object Set Transparency Event.
 *
 * This event is generated by the library to notify an object that its
 * transparency level has been changed.
 * The event is generated by:
 * MObjectSetTransparency(), MObjectSetTransparent(), MObjectSetSolid(),
 * CmObject::setTransparency, CmObject::setTransparent, CmObject::setSolid
 */
typedef struct {
/// Event type (E_SET_TRANSPARENCY)
	int type;
/// Transparency level (0-255)
	CARD8  level;
} MSetTransparencyEvent;

/**
 * \brief Union for MGui events.
 *
 */
union MEvent {
        int type;
        MAnyWindowEvent any_window_event;
        MKeyEvent key;
        MMouseEvent mouse;
        MExposeEvent expose;
        MWMCloseEvent wm_close;
        MFocusEvent focus;
        MUnmapEvent unmap;
        MMapEvent map;
        MConfigureEvent configure;
        MResizeEvent resize;
        MMoveEvent move;
        MSetColorEvent setcolor;
        MSetShadowEvent setshadow;
        MSetGetTextEvent setgettext;
        MSetSensitivity setsensitivity;
		MSetBGImageEvent setbgimage;
		MSetTransparencyEvent settransparency;
};
typedef union MEvent MEvent;
//@}


/** \ingroup macros
 * \name Predefined Mouse Cursors.
 * When setting a particular mouse cursor to be operative in a given MGui object
 * via MObjectSetCursor or CmObject::setCursor, you can use either one of the
 * predefined mouse cursors or an application created one (via MCreateCursor() or
 * CmCursor::CmCursor)
 */
//@{
#define MC_NONE       (MTCursor)0
#define MC_UL_ARROW   (MTCursor)1
#define MC_CROSSHAIR  (MTCursor)2
#define MC_HOURGLASS  (MTCursor)3
#define MC_HSIZE	  (MTCursor)4
#define MC_VSIZE	  (MTCursor)5
#define MC_NWSIZE	  (MTCursor)6
#define MC_NESIZE	  (MTCursor)7
//@}

/** \ingroup macros
 * \name Static fonts.
 * These are predefined font identifiers to be used to set object font either
 * at creation time or via MObjectSetFont() and CmObject::setFont.
 * Fonts can also be loaded via MLoadFont() and CmFont::CmFont.
 */
//@{
#define DEFAULT_FONT    0
#define FIXED_SMALL     1
#define FIXED_MEDIUM    2
#define FIXED_LARGE     3
#define HELV_SMALL      4
#define HELV_MEDIUM     5
#define TIMES_SMALL     6
#define TIMES_MEDIUM    7
#define HELV_LARGE      8
#define TIMES_LARGE     9
//@}

/**
 * Static fonts
 * WARNING: OBSOLETE
 */
#define SMALL_FONT      FIXED_SMALL
#define MEDIUM_FONT     FIXED_MEDIUM
#define LARGE_FONT      FIXED_LARGE
#define TIMES_N_FONT    TIMES_SMALL
#define TIMES_L_FONT    TIMES_MEDIUM
#define HELV_N_FONT     HELV_SMALL
#define HELV_L_FONT     HELV_MEDIUM

/** \ingroup macros
 * \name Alignment options.
 * The first 3 are used for text alignment in LABEL and text-only BUTTON
 * objects and in Text drawing primitives (CmPushButton::setAlignment).
 * All are also used to align objects in the TableForm cell boundaries
 * when a Form is enlarged (MObjectAlign(), MTableFormSetColAlign(),
 * MTableFormSetRowAlign(), CmObject::align, CmAbsTableForm::setColAlign,
 * CmAbsTableForm::setRowAlign).
 * Moreover they all can be used to set the
 * Text vs Image alignment in Buttons (Push/Toggle/Radio) having both
 * image and text via MButtonSetTextVsImageAlignment() and 
 * CmAbsButton::setTextVsImageAlignment.
 */
//@{
#define LEFT_ALIGN      0
#define RIGHT_ALIGN     1
#define CENTER_ALIGN    2
#define TOP_ALIGN	3
#define BOTTOM_ALIGN	4
#define UNDEF_ALIGN     0x3f
//@}

/** \ingroup macros
 * \name Object Options.
 * For use with MObjectEnableOptions() and MObjectSaveOptions().
 */
//@{
#define MO_FONT         0x1
#define MO_FGCOLOR      0x2
#define MO_BGCOLOR      0x4
#define MO_SHADOW       0x8
#define MO_TEXT         0x10
#define MO_BG_IMAGE     0x20
#define MO_TRANSPARENCY 0x40
#define MO_COLOR        (MO_FGCOLOR|MO_BGCOLOR)
#define MO_ALL          0x7f
//@}

/** \ingroup macros
 * \name Object background Image drawing modes.
 * To be used in MObjectSetBackgroundImage(), MObjectSetBackgroundImageFile()
 * CmObject::setBackgroundImage(), CmObject::setBackgroundImageFile()
 */
//@{
#define BI_TILED	0
#define BI_CENTERED	1
#define BI_SIZE_FORCING	2	/* Object size is forced to be equal to image size */
//@}


/** \ingroup macros
 * \name Object shadows.
 * To be used in MObjectSetShadow(), CmObject::setShadow()
 */
//@{
#define WS_NONE         0
#define WS_SHADOW_OUT   1
#define WS_SHADOW_IN    2
#define WS_ETCHED_IN    3
#define WS_ETCHED_OUT   4
//@}

/*
 * The following macros are present only for sources written for
 * release 1.4.3 or earlier. Don't use in new sources!
 */
#define WS_HATCHED_IN   WS_ETCHED_IN
#define WS_HATCHED_OUT  WS_ETCHED_OUT

/** \ingroup macros
 * \name Object Types.
 * Function MObjectType() and method CmObject::type return one these value
 * as the object type. Note that compound objects do not have specific values.
 */
//@{
#define OT_SHELL        1
#define OT_FORM         2
#define OT_LABEL        3
#define OT_PBUTTON      4
#define OT_TBUTTON      5
#define OT_RBUTTON      6
#define OT_SCROLLBAR    7
#define OT_EDIT         8
#define OT_STATUSBAR    9
#define OT_TEXT         10
#define OT_SBLIST       11
#define OT_MENUBAR      12
#define OT_MENU         13
#define OT_DRAWAREA     14
#define OT_SPLITTER     15
//@}

/**
  MNODE is the Node type identifier.
 */
#ifdef _MGTREE_H
	typedef struct _mtnode *MTNODE;
#else
	typedef void *MTNODE;
#endif


/** \ingroup macros
 * \name SHELL creation options. 
 * To be used as bitwise or-ed in the 'flags' parameter of MCreateShell(),
 * CmShell::CmShell().
 */
//@{
/** Internal use (MENU) */
#define SF_NO_FOCUS_CHANGE      0x1
/** The area under the appearing SHELL
 * window is stored for a fast redraw
 * when the window is deleted
 */
#define SF_SAVEUNDER            0x2
/** Input focus is grabbed by the SHELL
 * until it is removed
 */
#define SF_MODAL                0x4
/** Undecorated window look (MENU) */
#define SF_POPUP                0x8
/** SHELL ignores the CLOSE message
 * sent by the Window Manager
 */
#define SF_IGNORE_WM_CLOSE      0x10
/** The SHELL only unmaps itself
 * (not destroy) when receives the
 * CLOSE message from the Window Manager
 */
#define SF_UNMAP_ON_WM_CLOSE    0x20
/** No close button in the title bar */
#define SF_NO_CLOSE             0x40
/** No maximize button in the title bar*/
#define SF_NO_MAXIMIZE          0x80
/** No minimize button in the title bar*/
#define SF_NO_MINIMIZE          0x100
/** Resize disabled (both width and height) */
#define SF_NO_RESIZE            0x200
/** Width Resize disabled */
#define SF_NO_W_RESIZE          0x400
/** Height Resize disabled */
#define SF_NO_H_RESIZE          0x800
//@}

/** \ingroup macros
 * \name Shell position/size defaults.
 * To be used in MShellRealizeXY(), MShellRealizeXYWH(), CmShell::realize()
 */
//@{
/** Default x/y position useable in MShellRealizeXY() */
#define DEF_POSITION -32768
/* Default width/height useable in MShellRealizeXYWH() */
#define DEF_SIZE	 -32768
//@}

/** \ingroup macros
 * \name MTNODE creation flags.
 * To be used in MTreeCreateNode().
 */
//@{
#define NF_CONTAINER	0x1
#define NF_TOGGLE		0x2
//@}

/** \ingroup macros
 * \name DirBrowser creation flags.
 * To be used in MCreateDirBrowser().
 */
//@{
/** Allow renaming directories */
#define DBF_ALLOW_RENAME 0x1
/** Allow creating new directories */
#define DBF_ALLOW_CREATE 0x2
/** Allow deleting empty directories */
#define DBF_ALLOW_DELETE 0x4
//@}

/** \ingroup macros
 * \name SPOOL return values for MSpoolFile() and MSpoolTempFile().
 */
//@{
#define MSPOOL_OK               0
#define MSPOOL_FULL_QUEUE       -1
#define MSPOOL_NOT_READY        -2
#define MSPOOL_FILE_ERR         -3
//@}

/* Child Preferred Attachment */

#define F_PREF_ATT_POS  0
#define F_PREF_ATT_ROW  1
#define F_PREF_ATT_COL  2
#define F_PREF_ATT_TABLE 3

/** \ingroup macros
 *	\name Child Attachment
 */
//@{
#define F_ATT_NONE      0
#define F_ATT_FORM      1
#define F_ATT_OBJ       2
//@}

/** \ingroup structures 
 \brief DrawArea Object's resize data.

  This structure is passed as the object data in the resize callback of the 
  DrawArea object. The object uses it to notify the program that its size is
  being changed.
  See callback \ref DRAWAREA_PREF_CB for C API, and \ref DRAWAREAPREF_CB
  for C++ API.
  For the full DrawArea C API, see \ref drawarea. CmDrawArea for C++ API.
 */

struct DRAWAREA_PREF {
			/** Old width */
        int old_w;
			/** Old height */
        int old_h;
			/** New width */
        int new_w;
			/** New height */
        int new_h;
};
typedef struct DRAWAREA_PREF DRAWAREA_PREF;

/** \ingroup macros
 * \name SCROLLBAR TYPE.
 * For use in MCreateScrollBar(), CmScrollBar 
 */
//@{
#define SBT_HOR         0
#define SBT_VERT        1
//@}

/* SBLIST */
#define SBLT_TEXT       0
#define SBLT_LIST       1
#define SBLT_ELIST      2
#define SBLT_FILE       3
#define SBL_ITEM_SELECTED       0x1L
#define SBL_ITEM_CHECKED        0x2L

/** \ingroup structures 
 \brief List Item.

  Primarily used to supply data in virtual list Objects (Skip and Clip Lists).
  This structure is notified by Skip/Clip List object inside structure
  \ref SBL_NEW_BLOCK when the object queries new data to the application.
  See \ref list for a complete C API definition, and CmSList for C++ API.
 */
struct XL_ITEM {
/** Points to next item */
        struct XL_ITEM  *next;
/** Points to the item text */
        char            *data;
/** Internal use */
        long            misc;
/** Item's user data */
        void            *u_data;
};
typedef struct XL_ITEM XL_ITEM;

/** \ingroup structures 
 \brief List Item Activation.

  This structure is passed by the List Object in callback \ref LIST_ACT_CB (C API)
  and \ref LISTACT_CB (C++ API) to notify item activation.
  See \ref list for a complete C API definition, and CmSList for C++ API.
 */

struct LIST_ACT {
			/** Item text */
        char *item;
        		/** Item's user data */
        void *u_data;
        		/** Item's position in the list */
        long pos;
        			 /** Item's selection status flag */
        unsigned char selected;
        			 /** Item's check status flag */
        unsigned char checked;
};
typedef struct LIST_ACT LIST_ACT;

/** \ingroup structures 
 \brief List Item Selection.

  This structure is passed by the List Object to the Selection callback
  \ref LIST_SEL_CB (C API) and \ref LISTSEL_CB (C++ API) when the focused
  item changes.
  If the List owns the input focus and its last item is removed, then the
  callback is called with a value of -1 in the pos attribute.
  See \ref list for a complete C API definition, and CmSList for C++ API.
 */
struct LIST_SEL {
			/** Item text */
        char *item;
        		/** Item's user data */
        void *u_data;
        		/** Item's position in the list */
        long pos;
        			/** Item's selection status flag */
        unsigned char selected;
        			/** Item's check status flag */
        unsigned char checked;
        		/** Event that causes the call (mouse or keyboard).
                    	    NULL if the change is due to a DelItem
                    	*/
        MEvent *pe;
};
typedef struct LIST_SEL LIST_SEL;

/** \ingroup structures 
 \brief Template for EditList Object's creation.

  See MCreateEditList() and CmEditList for object creation details.
 */

struct EL_TEMPLATE {
			/** Field's head */
        char *head;
        		/** Field's length */
        short len;
        		/** Field's visible len */
        short vis_len;
        		/** Editable flag */
        short changeable;
        		/** Edit Object returned */
        MOBJECT edit;
        		/** Label heading returned */
        MOBJECT label;
};
typedef struct EL_TEMPLATE EL_TEMPLATE;

/** \ingroup structures 
 \brief Data feed for Skip and Clip List objects.

	This structure is passed to the NewBlock callback by the 'Clip' and
   'Skip' List Objects when, due to user scrolling, the objects needs to query
   the application to supply new data items.
   The member 'nr' is also a return value of the callback in case of
   'Skip' List Object. It specifies the number of rows actually returned.
  See MCreateSListSkip(), MCreateSListClip() and CmClippedSList for object creation details.
 */

struct SBL_NEW_BLOCK {
/** This field is meaningful only in case of 'Skip' List.
	The Skip List object sets this value to tell the application where to
	retrieve items from.
	Possible values:
	- SEEK_SET (from the beginning)
	- SEEK_CUR (from current position)
	- SEEK_END (from the end of the list)
*/
        int from;
/** Starting offset position relative to the location specified by 'from' */
        long offset;
/** Item length */
        int nc;
/** Number of rows (items) the application is asked to supply.
	If the nr of actually available items is smaller than the number requested
	by the object, then the application should set this field to the actual
	number of returned items.
*/
        int nr;
/** Pointer to destination item list (where the application can store returned items) */
        XL_ITEM *pi;
};
typedef struct SBL_NEW_BLOCK SBL_NEW_BLOCK;

/** \ingroup structures 
 \brief Used by the ItemSeek callback for 'Clip' and 'Skip' lists.

   Since Skip and Clip List objects normally contain only a subset of the items
   actually included in the virtual List, the application is supposed to help these
   objects in navigating the list when the user presses keys.
   The object notifies the substring that needs to be searched for by the
   application and the application returns back the found item position.
  See MCreateSListSkip(), MCreateSListClip() and CmClippedSList for object creation details.
 */

struct SBL_ITEM_SEEK {
			/** returned item's position (0 in case of Skip List, -1 if not found) */
        long pos;
        		/** substring to search item for */
        char *sub_item;
};
typedef struct SBL_ITEM_SEEK SBL_ITEM_SEEK;

/** \ingroup structures
	Utility macro to mark a List item (\ref XL_ITEM) as checked.
*/
#define XLItemCheck(pxl)        ((pxl)->misc |= SBL_ITEM_CHECKED)

/** \ingroup structures
	Utility macro to uncheck a List item (\ref XL_ITEM).
*/
#define XLItemUncheck(pxl)      ((pxl)->misc &= ~SBL_ITEM_CHECKED)

/** \ingroup macros
 * \name EDIT filter values.
 * Use in MEditSetFilter(), CmAbsEditField::setFilter
 */
//@{
#define NOCHAR          0
#define ANYCHAR         0x1
#define ALPHA           0x2
#define DIGIT           0x4
#define SPACE           0x8

/* Basic EDIT filter modifiers */
#define SIGNED          0x100
#define DOT             0x200
#define UPPER           0x400
#define LOWER           0x800

/* Composed EDIT filter values */
#define ALPHANUM        (ALPHA|DIGIT)
#define UPALPHA         (ALPHA|UPPER)
#define LOWALPHA        (ALPHA|LOWER)
#define U_INT            DIGIT
#define S_INT           (DIGIT|SIGNED)
#define U_FLOAT         (DIGIT|DOT)
#define S_FLOAT         (DIGIT|DOT|SIGNED)
//@}

/** \ingroup structures 
 \brief Cursor Position for the Edit Object.

   This structure is passed to the CursorPosition callbacks \ref EDIT_CURS_CB (C API)
   and \ref EDITCURS_CB (C++ API).
   \sa CmAbsEditWindow::setCursorCallback, MEditSetCursorPosCallback() 
 */

struct EDIT_CURS {
/** cursor position in the text stream */
        int pos;
/** cursor x position */
        int pos_x;
/** cursor y position */
        int pos_y;
};
typedef struct EDIT_CURS EDIT_CURS;

/** \ingroup structures 
 \brief Value Change for the Edit Object.

   The Edit Object passes this structure to the ValueChange callback
   \ref EDIT_CHANGE_CB (C API) and \ref EDITCHANGE_CB (C++ API) when its text changes. Last member is a return flag.
   \sa CmAbsEdit::setValueChangeCallback, MEditSetValueChangeCallback() 
 */

struct EDIT_CHANGE {
/** EDIT text before the change */
        char *current_text;
/** New text */
        char *change;
/** New text insert position */
        int pos;
/** Current text's lenght to substitute */
        int len;
/** New text's lenght */
        int change_len;
/** Validation flag */
        unsigned validate;
/** Reject the change if set to False */
        unsigned exec;
};
typedef struct EDIT_CHANGE EDIT_CHANGE;

/** \ingroup structures 
 \brief Validation data for the EditField Object.

  The EDIT Object passes this structure to the Validation callbacks
  \ref EDIT_VAL_CB (C API) and \ref EDITVAL_CB (C++ API) when it's about to lose the input focus.
  Last three members are return flags.
   \sa CmAbsEditField::setValidateCallback, MEditSetValCallback() 
 */
struct EDIT_VAL {
/** EDIT text */
        char *text;
/** Text lenght */
        short len;
/** Key code originator (0 if not applicable) */
        int key;
/** If True, EDIT will lose the focus anyway */
        unsigned focus_lost;
/** If set to False, EDIT will keep focus */
        unsigned focus_leave;
/** If set to True EDIT redraws itself */
        unsigned text_changed;
/** If set to True EDIT becomes validated */
        unsigned text_validated;
};
typedef struct EDIT_VAL EDIT_VAL;

/** \ingroup macros
 * \name Socket Events
 */
//@{
#define SE_READ		0x1
#define SE_WRITE	0x2
#define SE_EXCP		0x4
//@}
typedef void (*SOCKET_CB)(int s, unsigned event, void *data);


/* MENU */

typedef void *MENU_ITEM;


typedef CARD32 TIMEOUT_ID;
typedef INT32 MVALUE;

/**
 *	\defgroup c_cb C Callback types
 */
//@{
typedef void (*TIMEOUT_CB)(TIMEOUT_ID id, void *data);
typedef int (*STRCMP_CB)(const char *, const char *);

#ifdef NO_MGUI_CB_PROTO
typedef void (*MGUI_CB)();
#else
typedef void (*MGUI_CB)(MOBJECT obj, void *od, void *ad);
#endif

#if defined UNIQUE_CB_PROTO || defined NO_MGUI_CB_PROTO
typedef MGUI_CB OBJ_EVENT_CB;
typedef MGUI_CB OBJ_STRING_CB;
typedef MGUI_CB OBJ_VALUE_CB;
typedef MGUI_CB WM_CLOSE_CB;
typedef MGUI_CB NEW_BLOCK_CB;
typedef MGUI_CB ITEM_SEEK_CB;
typedef MGUI_CB EDIT_VAL_CB;
typedef MGUI_CB EDIT_CURS_CB;
typedef MGUI_CB EDIT_CHANGE_CB;
typedef MGUI_CB LIST_ACT_CB;
typedef MGUI_CB LIST_SEL_CB;
typedef MGUI_CB DRAWAREA_PREF_CB;
typedef MGUI_CB MENU_ITEM_CB;
typedef MGUI_CB MTREE_NODE_CB;
typedef MGUI_CB MTREE_POPUP_CB;
#else
typedef void (*OBJ_EVENT_CB)(MOBJECT obj, MEvent *e, void *ad);
typedef void (*OBJ_STRING_CB)(MOBJECT obj, char *str, void *ad);
typedef void (*OBJ_VALUE_CB)(MOBJECT obj, MVALUE value, void *ad);
typedef void (*WM_CLOSE_CB)(MOBJECT obj, void *odata, void *cdata);
typedef void (*NEW_BLOCK_CB)(MOBJECT obj, SBL_NEW_BLOCK *nbs, void *ad);
typedef void (*ITEM_SEEK_CB)(MOBJECT obj, SBL_ITEM_SEEK *iss, void *ad);
typedef void (*EDIT_VAL_CB)(MOBJECT edit, EDIT_VAL *ev, void *ad);
typedef void (*EDIT_CURS_CB)(MOBJECT edit, EDIT_CURS *ev, void *ad);
typedef void (*EDIT_CHANGE_CB)(MOBJECT edit, EDIT_CHANGE *ec, void *ad);
typedef void (*LIST_ACT_CB)(MOBJECT list, LIST_ACT *la, void *ad);
typedef void (*LIST_SEL_CB)(MOBJECT list, LIST_SEL *la, void *ad);
typedef void (*DRAWAREA_PREF_CB)(MOBJECT drawa, DRAWAREA_PREF *dp, void *ad);
typedef void (*MENU_ITEM_CB)(MENU_ITEM item, MVALUE status, void *ad);
typedef void (*MTREE_NODE_CB)(MTNODE node, void *ad);
typedef void (*MTREE_POPUP_CB)(MTNODE node, int mx, int my, void *ad);
typedef int  (*MTREE_NODE_RENAME_CB)(MTNODE node, const char *, void *ad);
typedef int  (*MTREE_NODE_DELETE_CB)(MTNODE node, void *ad);
typedef void (*DB_DIR_CHANGE_CB)(MOBJECT, const char *, void *);
typedef int  (*DB_ACTION_CB)(MOBJECT, const char *, char *, void *);
#endif /* UNIQUE_CB_PROTO */
//@}

/** \ingroup structures 
 \brief File Browser Preferences. Used by the Utility functions MFileBowserExt() and MDirBowserExt().

*/
struct FB_PREFS {
/** If True the program's current working directory will be set on exit
to the last selected one */
  CARD8 change_cwd;
/** If True the user will be allowed to create new directories via a popup menu */
  CARD8 allow_dir_create;
/** If True the user will be allowed to rename existing directories via a popup
menu or a mouse click on the selected directory */
  CARD8 allow_dir_rename;
/** If True the user will be allowed to delete empty directories via a popup
menu or by pressing the DEL key on the selected directory */
  CARD8 allow_dir_delete;
/** MTree directory object is by default solid for drawing efficiency reason */
  CARD8 transparent_tree;
/** Specifies alternative text font for tree, list and edit objects */
  MTFont font;
/** Specifies alternative text font for buttons */
  MTFont button_font;
/** Specifies alternative string to describe the file filter */
  const char *filter_label;
/** Specifies alternative string to describe the file name */
  const char *file_label;
/** Specifies alternative string to describe the OK button */
  const char *ok_button;
/** Specifies alternative string to describe the Close button */
  const char *close_button;
/** Specifies alternative string to describe the Create popup menu item */
  const char *create_menu;
/** Specifies alternative string to describe the Rename popup menu item */
  const char *rename_menu;
/** Specifies alternative string to describe the Delete popup menu item */
  const char *remove_menu;
/** Specifies alternative background image */
  MIMAGE bg_image;
/** Specifies alternative background image file (applied if bg_image is NULL) */
  const char *bg_image_file;
};
typedef struct FB_PREFS FB_PREFS;

/*
 *  Graphic printing currently works only under Windows
 */

#ifdef WIN_MGUI

/**
 * Macros for specifying the page mode
 */
#define PM_FULL		0	/* the printing is maximized */
#define PM_MM		1	/* dimesnsions in mm */
#define PM_PIXEL	3	/* dimensions in printer pixels */
#define PM_PERC		4	/* dimensions in percentage (0-100%) of the paper size */

/**
 * Macros for specifying the paper orientation
 */
#define PO_DEFAULT	 0	/* use the printer setup default */
#define PO_PORTRAIT  1  /* portrait orientation */
#define PO_LANDSCAPE 2  /* landscape orientation */

typedef struct
{
	int mode;	/* Page mode one of PM_FULL, PM_MM, PM_PIXEL, PM_PERC */
	int x;		/* X position of the object to print (-1 to center horiz) */
	int y;		/* Y position of the object to print (-1 to center vert) */
	int w;		/* Width of the object to print */
	int h;		/* Height of the object to print */
} MPRINT_OPTIONS;

typedef struct
{
	int startPage;	/* Starting page to print */
	int endPage;	/* Ending page to print */
	int minPage;	/* min printable page */
	int maxPage;	/* max printable page */
} MPAGE_OPTIONS;

typedef void *DOC_ID;

#endif /* WIN_MGUI */
#endif
