
/*
 *    File mtree.c
 *      Demonstrates the use of the MTREE Object
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "mguipp.h"
#include "mguitest.hpp"

class MTreeDialog:public CmCallback
{
  CmShell *shell;
  CmSList *slist;
  CmEditField *edit;
  CmTree *tree;
  CmTreeNode *popup_node;
  CmMenu *menu;
public:
    MTreeDialog ();
  void nodeSelection (CmTreeNode * node);
  void addNode (void);
  void destroyNode (void);
  void nodePopupCB (CmTreeNode *node, int mx, int my);
  void createCB (int status);
  void renameCB (int status);
  void removeCB (int status);
  int  nodeRenameCB (CmTreeNode *node, const char *oldname);
  int  nodeDeleteCB (CmTreeNode *node);
  void closeDialog (void)
  {
    delete shell;
    delete this;
  };
};


void
MTreeDialog::nodeSelection (CmTreeNode * node)
{
  char text[256];

  slist->clear ();
  node = node->getFirstChild ();
  while (node)
  {
    node->getText (text);
    slist->addItem (text, NULL);
    node = node->getNext ();
  }
}

void
MTreeDialog::addNode (void)
{
  char text[64];

  edit->getText (text);
  new CmTreeNode (tree, tree->getSelectedNode (), 0, NF_CONTAINER, text);
}

void
MTreeDialog::destroyNode (void)
{
  CmTreeNode *node = tree->getSelectedNode ();

  if (node)
    delete node;
}

void
MTreeDialog::createCB (int status)
{
  CmTreeNode *node;
  popup_node->open (True);
  node = new CmTreeNode (popup_node->getMTree(), popup_node, 0, NF_CONTAINER, "New");
  node->edit ();
}

void
MTreeDialog::renameCB (int status)
{
  popup_node->edit ();
}

void
MTreeDialog::removeCB (int status)
{
  delete popup_node;
}

void
MTreeDialog::nodePopupCB (CmTreeNode *node, int mx, int my)
{
  popup_node = node;
  menu->popup (mx, my);
}

int
MTreeDialog::nodeRenameCB (CmTreeNode *node, const char *oldname)
{
  return 1;
}

int
MTreeDialog::nodeDeleteCB (CmTreeNode *node)
{
  return 1;
}

MTreeDialog::MTreeDialog ()
{
  CmColForm *cform;
  CmSFile *sfile;
  CmPushButton *pb;
  CmTreeNode *first, *second, *third, *fourth, *one, *two, *four, *last;
  CmImage *closed, *opened, *selected, *leaf;

  shell = new CmShell ("MTree", SF_MODAL);
  shell->setWMCloseCallback (this, (VOID_CB) & MTreeDialog::closeDialog);

  cform = new CmColForm (shell);
/*
 * Create the MTree Object
 */
  closed = new CmImage ("fbcfold.bmp");
  closed->setMaskByPixel (0, 0);
  closed->install ();
  opened = new CmImage ("fbofold.bmp");
  opened->setMaskByPixel (0, 0);
  opened->install ();
  selected = new CmImage ("fbsfold.bmp");
  selected->setMaskByPixel (0, 0);
  selected->install ();
  leaf = new CmImage ("fdisk.bmp");
  leaf->setMaskByPixel (0, 0);
  leaf->install ();

  tree = new CmTree (cform, HELV_MEDIUM, 160, 160, 2);
  tree->setBackgroundImageFile ("tile5.bmp", BI_TILED);

  slist = new CmSList (cform, HELV_MEDIUM, 48, 12, 12);

  tree->setImages (0, closed, opened, selected);
  tree->setImages (1, leaf, NULL, NULL);
  tree->setSelectCallback (this, (MTREENODE_CB) & MTreeDialog::nodeSelection);
  tree->setRenameCallback (this, (MTREERENAME_CB) & MTreeDialog::nodeRenameCB);
  tree->setDeleteCallback (this, (MTREEDELETE_CB) & MTreeDialog::nodeDeleteCB);

  menu = new CmMenu (HELV_SMALL);
  new CmMenuItem (menu, "New node ...", this, (MENUITEM_CB) & MTreeDialog::createCB);
  new CmMenuItem (menu, "Rename node ...", this, (MENUITEM_CB) & MTreeDialog::renameCB);
  new CmMenuItem (menu, "Delete node", this, (MENUITEM_CB) & MTreeDialog::removeCB);

  tree->setPopupCallback (this, (MTREEPOPUP_CB) & MTreeDialog::nodePopupCB);

  first = new CmTreeNode (tree, NULL, 0, NF_CONTAINER, "First Node");
  second = new CmTreeNode (tree, NULL, 0, NF_CONTAINER, "Second Node");
  third = new CmTreeNode (tree, NULL, 0, NF_CONTAINER, "Third Node");
  fourth = new CmTreeNode (tree, NULL, 0, NF_CONTAINER, "Fourth Node");

  one = new CmTreeNode (tree, first, 0, NF_CONTAINER, "First Child");
  last = new CmTreeNode (tree, one, 1, 0, "Second Child");
  last = new CmTreeNode (tree, one, 1, 0, "Last Child");
  one = new CmTreeNode (tree, first, 0, NF_CONTAINER, "Another Child");
  two = new CmTreeNode (tree, second, 0, NF_CONTAINER, "Second Child");
  four = new CmTreeNode (tree, fourth, 0, NF_CONTAINER, "Fourth Child");
  last = new CmTreeNode (tree, four, 1, 0, "Last Child");

  cform = new CmColForm (shell);

  edit = new CmEditField (cform, "new node text", 32, HELV_MEDIUM);

  pb = new CmPushButton (cform, "&Add Node", TIMES_MEDIUM);
  pb->setCallback (this, (VOID_CB) & MTreeDialog::addNode);

  pb = new CmPushButton (cform, "&Destroy Node", TIMES_MEDIUM);
  pb->setCallback (this, (VOID_CB) & MTreeDialog::destroyNode);

/*
 * Create a SFILE Object to show this source file in the dialog
 */
  sfile = new CmSFile (shell, __FILE__, FIXED_MEDIUM, 10, 48);
/*
 * This PUSH BUTTON closes the dialog
 */
  pb = new CmPushButton (shell, "Close", TIMES_MEDIUM);
  pb->setCallback (this, (VOID_CB) & MTreeDialog::closeDialog);

  shell->realize ();
  last->select (False);
}

/*****************************************************
 *	This callback opens the dialog when the
 *	corresponding menu item is selected
 *****************************************************/
void
MTreeDemo (void)
{
  new MTreeDialog ();
}
